@@ -9,13 +9,17 @@ from mypy.nodes import Context
 
 
 def apply_generic_arguments(callable: CallableType, orig_types: Sequence[Optional[Type]],
-                            msg: MessageBuilder, context: Context) -> CallableType:
+                            msg: MessageBuilder, context: Context,
+                            skip_unsatisfied: bool = False) -> CallableType:
     """Apply generic type arguments to a callable type.
 
     For example, applying [int] to 'def [T] (T) -> T' results in
     'def (int) -> int'.
 
     Note that each type can be None; in this case, it will not be applied.
+
+    If `skip_unsatisfied` is True, then just skip the types that don't satisfy type variable
+    bound or constraints, instead of giving an error.
     """
     tvars = callable.variables
     assert len(tvars) == len(orig_types)
@@ -25,7 +29,9 @@ def apply_generic_arguments(callable: CallableType, orig_types: Sequence[Optiona
     for i, type in enumerate(types):
         assert not isinstance(type, PartialType), "Internal error: must never apply partial type"
         values = callable.variables[i].values
-        if values and type:
+        if type is None:
+            continue
+        if values:
             if isinstance(type, AnyType):
                 continue
             if isinstance(type, TypeVarType) and type.values:
@@ -34,15 +40,31 @@ def apply_generic_arguments(callable: CallableType, orig_types: Sequence[Optiona
                 if all(any(is_same_type(v, v1) for v in values)
                        for v1 in type.values):
                     continue
+            matching = []
             for value in values:
                 if mypy.subtypes.is_subtype(type, value):
-                    types[i] = value
-                    break
+                    matching.append(value)
+            if matching:
+                best = matching[0]
+                # If there are more than one matching value, we select the narrowest
+                for match in matching[1:]:
+                    if mypy.subtypes.is_subtype(match, best):
+                        best = match
+                types[i] = best
             else:
-                msg.incompatible_typevar_value(callable, type, callable.variables[i].name, context)
-        upper_bound = callable.variables[i].upper_bound
-        if type and not mypy.subtypes.is_subtype(type, upper_bound):
-            msg.incompatible_typevar_value(callable, type, callable.variables[i].name, context)
+                if skip_unsatisfied:
+                    types[i] = None
+                else:
+                    msg.incompatible_typevar_value(callable, type, callable.variables[i].name,
+                                                   context)
+        else:
+            upper_bound = callable.variables[i].upper_bound
+            if not mypy.subtypes.is_subtype(type, upper_bound):
+                if skip_unsatisfied:
+                    types[i] = None
+                else:
+                    msg.incompatible_typevar_value(callable, type, callable.variables[i].name,
+                                                   context)
 
     # Create a map from type variable id to target type.
     id_to_type = {}  # type: Dict[TypeVarId, Type]
