@@ -9,9 +9,12 @@ from mypy.scope import Scope
 from mypy.options import Options
 from mypy.version import __version__ as mypy_version
 
+MYPY = False
+if MYPY:
+    from typing_extensions import Final
 
 T = TypeVar('T')
-allowed_duplicates = ['@overload', 'Got:', 'Expected:']
+allowed_duplicates = ['@overload', 'Got:', 'Expected:']  # type: Final
 
 
 class ErrorInfo:
@@ -402,10 +405,10 @@ class Errors:
                                                                      str, str]]:
         """Translate the messages into a sequence of tuples.
 
-        Each tuple is of form (path, line, col, message.  The rendered
-        sequence includes information about error contexts. The path
-        item may be None. If the line item is negative, the line
-        number is not defined for the tuple.
+        Each tuple is of form (path, line, col, severity, message).
+        The rendered sequence includes information about error contexts.
+        The path item may be None. If the line item is negative, the
+        line number is not defined for the tuple.
         """
         result = []  # type: List[Tuple[Optional[str], int, int, str, str]]
         # (path, line, column, severity, message)
@@ -612,4 +615,5 @@ def report_internal_error(err: Exception, file: Optional[str], line: int,
         print('{}: note: use --pdb to drop into pdb'.format(prefix), file=sys.stderr)
 
     # Exit.  The caller has nothing more to say.
-    raise SystemExit(1)
+    # We use exit code 2 to signal that this is no ordinary error.
+    raise SystemExit(2)
