@@ -344,6 +344,8 @@ class StrConv(NodeVisitor[str]):
 
     def visit_name_expr(self, o: 'mypy.nodes.NameExpr') -> str:
         pretty = self.pretty_name(o.name, o.kind, o.fullname, o.is_inferred_def, o.node)
+        if isinstance(o.node, mypy.nodes.Var) and o.node.is_final:
+            pretty += ' = {}'.format(o.node.final_value)
         return short_type(o) + '(' + pretty + ')'
 
     def pretty_name(self, name: str, kind: Optional[int], fullname: Optional[str],
@@ -355,8 +357,10 @@ class StrConv(NodeVisitor[str]):
             id = self.format_id(target_node)
         else:
             id = ''
-        if kind == mypy.nodes.GDEF or (fullname != name and
-                                       fullname is not None):
+        if isinstance(target_node, mypy.nodes.MypyFile) and name == fullname:
+            n += id
+        elif kind == mypy.nodes.GDEF or (fullname != name and
+                                         fullname is not None):
             # Append fully qualified name for global references.
             n += ' [{}{}]'.format(fullname, id)
         elif kind == mypy.nodes.LDEF:
@@ -438,7 +442,7 @@ class StrConv(NodeVisitor[str]):
         return self.dump([o.base, o.index], o)
 
     def visit_super_expr(self, o: 'mypy.nodes.SuperExpr') -> str:
-        return self.dump([o.name], o)
+        return self.dump([o.name, o.call], o)
 
     def visit_type_application(self, o: 'mypy.nodes.TypeApplication') -> str:
         return self.dump([o.expr, ('Types', o.types)], o)
