@@ -32,15 +32,16 @@ class PythonCmdlineSuite(DataSuite):
     native_sep = True
 
     def run_case(self, testcase: DataDrivenTestCase) -> None:
-        test_python_cmdline(testcase)
+        for step in [1] + sorted(testcase.output2):
+            test_python_cmdline(testcase, step)
 
 
-def test_python_cmdline(testcase: DataDrivenTestCase) -> None:
+def test_python_cmdline(testcase: DataDrivenTestCase, step: int) -> None:
     assert testcase.old_cwd is not None, "test was not properly set up"
     # Write the program to a file.
     program = '_program.py'
     program_path = os.path.join(test_temp_dir, program)
-    with open(program_path, 'w') as file:
+    with open(program_path, 'w', encoding='utf8') as file:
         for s in testcase.input:
             file.write('{}\n'.format(s))
     args = parse_args(testcase.input[0])
@@ -75,30 +76,39 @@ def test_python_cmdline(testcase: DataDrivenTestCase) -> None:
         # Ignore stdout, but we insist on empty stderr and zero status.
         if err or result:
             raise AssertionError(
-                'Expected zero status and empty stderr, got %d and\n%s' %
-                (result, '\n'.join(err + out)))
+                'Expected zero status and empty stderr%s, got %d and\n%s' %
+                (' on step %d' % step if testcase.output2 else '',
+                 result, '\n'.join(err + out)))
         for path, expected_content in testcase.output_files:
             if not os.path.exists(path):
                 raise AssertionError(
-                    'Expected file {} was not produced by test case'.format(path))
-            with open(path, 'r') as output_file:
+                    'Expected file {} was not produced by test case{}'.format(
+                        path, ' on step %d' % step if testcase.output2 else ''))
+            with open(path, 'r', encoding='utf8') as output_file:
                 actual_output_content = output_file.read().splitlines()
             normalized_output = normalize_file_output(actual_output_content,
                                                       os.path.abspath(test_temp_dir))
-            if testcase.suite.native_sep and os.path.sep == '\\':
-                normalized_output = [fix_cobertura_filename(line) for line in normalized_output]
-            normalized_output = normalize_error_messages(normalized_output)
+            # We always normalize things like timestamp, but only handle operating-system
+            # specific things if requested.
+            if testcase.normalize_output:
+                if testcase.suite.native_sep and os.path.sep == '\\':
+                    normalized_output = [fix_cobertura_filename(line)
+                                         for line in normalized_output]
+                normalized_output = normalize_error_messages(normalized_output)
             assert_string_arrays_equal(expected_content.splitlines(), normalized_output,
-                                       'Output file {} did not match its expected output'.format(
-                                           path))
+                                       'Output file {} did not match its expected output{}'.format(
+                                           path, ' on step %d' % step if testcase.output2 else ''))
     else:
-        out = normalize_error_messages(err + out)
+        if testcase.normalize_output:
+            out = normalize_error_messages(err + out)
         obvious_result = 1 if out else 0
         if obvious_result != result:
             out.append('== Return code: {}'.format(result))
-        assert_string_arrays_equal(testcase.output, out,
-                                   'Invalid output ({}, line {})'.format(
-                                       testcase.file, testcase.line))
+        expected_out = testcase.output if step == 1 else testcase.output2[step]
+        assert_string_arrays_equal(expected_out, out,
+                                   'Invalid output ({}, line {}){}'.format(
+                                       testcase.file, testcase.line,
+                                       ' on step %d' % step if testcase.output2 else ''))
 
 
 def parse_args(line: str) -> List[str]:
