@@ -3,7 +3,8 @@
 from typing import AbstractSet, Dict, Set, List
 
 from mypy.test.helpers import assert_equal, Suite
-from mypy.build import BuildManager, State, BuildSourceSet, SearchPaths
+from mypy.build import BuildManager, State, BuildSourceSet
+from mypy.modulefinder import SearchPaths
 from mypy.build import topsort, strongly_connected_components, sorted_components, order_ascc
 from mypy.version import __version__
 from mypy.options import Options
@@ -50,6 +51,7 @@ class GraphSuite(Suite):
             options=options,
             version_id=__version__,
             plugin=Plugin(options),
+            plugins_snapshot={},
             errors=errors,
             flush_errors=lambda msgs, serious: None,
             fscache=fscache,
